use std::sync::Arc;
use crate::domain::create_pokemon::Response;
use crate::repo::pokemon::{FetchAllError, Repository};

pub enum Error {
    Unknown,
}


pub fn execute(repo: Arc<dyn Repository>) -> Result<Vec<Response>, Error>{
    match repo.fetch_all() {
        Ok(pokemons) => Ok(pokemons.into_iter().map(|p| Response {
            number: u16::from(p.number),
            name: String::from(p.name),
            types: Vec::<String>::from(p.types),
        }).collect()),
        Err(FetchAllError::Unknown) => Err(Error::Unknown),
    }
}


#[cfg(test)]
mod tests {
    use std::sync::Arc;
    use crate::domain::entity::{PokemonName, PokemonNumber, PokemonTypes};
    use crate::domain::fetch_all_pokemons::{Error};
    use crate::domain::fetch_all_pokemons::execute;
    use crate::repo::pokemon::{InMemoryRepository, Repository};

    #[test]
    fn it_should_return_an_unkown_error_when_an_unexpected_error_happens() {
        let repo = Arc::new(InMemoryRepository::new().with_error());
        let res = execute(repo);
        match res {
            Err(Error::Unknown) => {},
            _ => unreachable!(),
        }
    }
    #[test]
    fn it_should_return_all_pokemons_ordered_by_increasing_number_otherwise() {
        let repo = Arc::new(InMemoryRepository::new());
        repo.insert(
            PokemonNumber::pikachu(),
            PokemonName::pikachu(),
            PokemonTypes::pikachu(),
        ).ok();
        repo.insert(
            PokemonNumber::charmander(),
            PokemonName::charmander(),
            PokemonTypes::charmander(),
        ).ok();
        let res = execute(repo);
        match res {
            Ok(res) => {
                assert_eq!(res[0].number, u16::from(PokemonNumber::charmander()));
                assert_eq!(res[0].name, String::from(PokemonName::charmander()));
                assert_eq!(res[0].types, Vec::<String>::from(PokemonTypes::charmander()));
                assert_eq!(res[1].number, u16::from(PokemonNumber::pikachu()));
                assert_eq!(res[1].name, String::from(PokemonName::pikachu()));
                assert_eq!(res[1].types, Vec::<String>::from(PokemonTypes::pikachu()));
            },
            Err(Error::Unknown) => {}
        }
    }
}